'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _errorTransformer = require('./errorTransformer');

var _errorTransformer2 = _interopRequireDefault(_errorTransformer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var someString = 'hello';
var rethrowErrorHandler = function rethrowErrorHandler() {};

afterEach(function () {
	_errorTransformer.TraceKit.report = _errorTransformer.TraceKit.fallback || _errorTransformer.TraceKit.report;
});

describe('ErrorTransformer', function () {
	describe('listener', function () {
		it('should be function on minimum config', function () {
			expect(_typeof((0, _errorTransformer2.default)())).toBe('function');
		});

		it('should call successHandler internally', function () {
			var successHandler = function successHandler(t) {
				return expect(t).toBe(someString);
			};
			(0, _errorTransformer2.default)('', {
				successHandler: successHandler,
				fetchOptions: { response: { text: function text() {
							return someString;
						}, ok: true } }
			}, { rethrowErrorHandler: rethrowErrorHandler })();
		});

		it('should call failedReportHandler internally', function () {
			var failedReportHandler = function failedReportHandler(e) {
				expect(e.message).toBe(someString);
			};
			(0, _errorTransformer2.default)('', {
				failedReportHandler: failedReportHandler,
				fetchOptions: { response: someString }
			}, { rethrowErrorHandler: rethrowErrorHandler })();
		});

		it('should return payload', function () {
			var returnedData = (0, _errorTransformer2.default)('', {
				fetchOptions: { response: { text: function text() {
							return someString;
						}, ok: true } }
			}, { rethrowErrorHandler: rethrowErrorHandler })({ some: someString });

			expect(returnedData).toMatchObject({ some: someString });
		});

		it('should transform payload', function () {
			var expected = { a: 'b' };
			var returnedData = (0, _errorTransformer2.default)('', {
				payloadMiddleware: function payloadMiddleware() {
					return expected;
				},
				fetchOptions: { response: { text: function text() {
							return null;
						}, ok: true } }
			}, { rethrowErrorHandler: rethrowErrorHandler })();
			expect(returnedData).toMatchObject(expected);
		});
	});

	it('should patch TraceKit_ so it calls rethrowErrorHandler internally', function () {
		// given:
		var report = function report(error) {
			var message = void 0;
			try {
				_errorTransformer.TraceKit.report(error);
			} catch (e) {
				message = e;
			}
			return message;
		};
		// then:
		expect(report(new Error(someString)).message).toBe(someString);
		expect(report(someString)).toBe(someString);

		// when:
		(0, _errorTransformer2.default)('', {}, { rethrowErrorHandler: rethrowErrorHandler });
		// then:
		expect(report()).toBe(undefined);
	});

	it('should call failedTryHandler internally', function () {
		var failedTryHandler = function failedTryHandler(r, sr, p, to, c) {
			expect(p).toBe(someString);
			sr(p, to, c + 1);
		};
		(0, _errorTransformer2.default)('', {
			failedTryHandler: failedTryHandler,
			retryCount: 5,
			fetchOptions: { response: { ok: false } }
		}, { rethrowErrorHandler: rethrowErrorHandler })(someString);
	});
});