'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.sendReport = sendReport;
function sendReport(payload, transportOpts) {
	var attempt = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
	var send = transportOpts.send,
	    payloadMiddleware = transportOpts.payloadMiddleware,
	    fetchOptions = transportOpts.fetchOptions,
	    successHandler = transportOpts.successHandler,
	    retryCount = transportOpts.retryCount,
	    failedTryHandler = transportOpts.failedTryHandler,
	    failedReportHandler = transportOpts.failedReportHandler;

	var transformedPayload = payloadMiddleware(payload);

	send(transformedPayload, fetchOptions).then(successHandler).catch(function (errorResponse) {
		if (attempt < retryCount) {
			failedTryHandler(errorResponse, sendReport, payload, transportOpts, attempt);
		} else {
			failedReportHandler(errorResponse, payload, transportOpts);
		}
	});
	return transformedPayload;
}

function defaultPayloadMiddleware(payload) {
	return _extends({
		time: new Date()
	}, payload);
}

var defaultHandlers = {
	success: function success(response) {
		console.info('Logging: reported', response);
	},
	failedTry: function failedTry(error, report, payload, transportOpts, attempt) {
		setTimeout(function () {
			report(payload, transportOpts, attempt + 1);
		}, transportOpts.retryTimeout);
		console.warn('Logging: Looks like logging host is unreachable, ' + ('retrying in ' + transportOpts.retryTimeout / 1000 + ' seconds'));
	},
	failedReport: function failedReport(error) {
		console.error('Logging: unable to send reports', error);
	}
};

function getFetchPayload(payload) {
	var otherFetchOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

	return Object.assign({
		method: 'POST',
		headers: { 'Content-Type': 'application/json' }
	}, _extends({
		body: JSON.stringify({ '@message': payload })
	}, otherFetchOptions));
}

function shouldBeOk(response) {
	if (!response.ok) {
		throw new Error(response);
	}
	return response;
}

function getDefault(url) {
	return function (payload) {
		var fetchOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
		return fetch(url, getFetchPayload(payload, fetchOptions)).then(shouldBeOk).then(function (response) {
			return response.text();
		});
	};
}

var getDefaultTransport = exports.getDefaultTransport = function getDefaultTransport(url) {
	return {
		send: getDefault(url),
		successHandler: defaultHandlers.success,
		failedTryHandler: defaultHandlers.failedTry,
		failedReportHandler: defaultHandlers.failedReport,
		payloadMiddleware: defaultPayloadMiddleware,
		retryCount: 2,
		retryTimeout: 60 * 1000
	};
};